/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *  CSTACK.C  --   stack class definitions                   *
 *							     *
 *  Language:      Borland C                                 *
 *  Platform:      Gateway 386/25, MS-DOS 5.0                *
 *  Application:   CSE 687                                   *
 *  Author:        Jim Fawcett, CST 2-127, (315) 443-4454    *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "cstack.h"        /* declare stack structure        */

/*----< int* init_stack(struct stack *s, int n) >------------*/
/*      allocate storage                                     */

void init_stack(struct stack *s, int n) {

  int *ptr;
  ptr = (int *) malloc(sizeof(int)*n);
  if (!ptr) {
    fprintf(stderr,"dynamic memory exhausted\n");
    exit(1);
  }
  s->stk  = ptr;
  s->size = n;
  s->top  = 0;
}

/*----< void push(struct stack *s, int datum) >--------------*/
/*      push integer datum on stack                          */

void push(struct stack *s, int datum) {

  if (s->top > s->size-1)
    fprintf(stderr,
	    "attempt to push beyond stack limits - datum lost\n");
  else
    s->stk[s->top++] = datum;
}

/*----< int pop(struct stack *s) >---------------------------*/
/*      remove integer datum from top of stack               */

int pop(struct stack *s) {

  if (s->top <= 0) {
    fprintf(stderr,
	    "attempt to pop datum from empty stack - zero returned\n");
    return 0;
  }
  else
    return s->stk[--(s->top)];
}
/**/
/*----< void show_stack(struct stack *s) >-------------------*/
/*      put stack contents on stdout                         */

void show_stack(struct stack *s) {

  #define step 20
  int i;
  for (i=0; i<s->top; i++) {
    fprintf(stdout,"%d  ",s->stk[i]);
    if (i!=0 && i%step)
      fprintf(stdout,"\n");
  }
}

/*----< int scan(struct stack *s, int datum) >---------------*/
/*      report depth at which datum is found on stack        */
/*      report -1 if not found                               */

int scan(struct stack *s, int datum) {

  int i;
  for (i=0; i<s->top; i++)
    if (s->stk[i] == datum)
      return i;
  return -1;
}

/**/
/*----< test stub >------------------------------------------*/

#ifdef test_stack
#include "cstack.h"      /* needed in separate client module */

int main(int argc, char *argv[]) {

  #define MAXSIZE 512
  char line[MAXSIZE];       /* buffer to capture input lines */
  struct stack instack;     /* stack used to reverse lines   */
  struct stack outstack;    /* stack used to reverse again   */

/*---- Note well: ---------------------------------------------

  We could declare pointers to instack and outstack like this:

    struct stack *instack_ptr;
    struct stack *outstack_ptr;

  but then we must also declare the stacks like this:

    struct stack instack;
    struct stack outstack;

  and set the pointers to point there:

    instack_ptr  = &instack;
    outstack_ptr = &outstack;

-------------------------------------------------------------*/

  FILE *fptr;
  int  len, i, ch = 0;

  if (argc < 2) {
    fprintf(stderr,"please enter test filename on command line\n");
    return 1;
  }
  if ((fptr = fopen(argv[1],"r")) == '\0') {
    fprintf(stderr, "can't open %s\n", argv[1]);
    return 1;
  }
  fprintf(stdout,"\nTesting C Stack\n\n");
  init_stack(&instack,MAXSIZE);
  init_stack(&outstack,MAXSIZE);
  while(ch != EOF) {
    len = 0;
  /* collect line from input stream */
    while ((ch = line[len] = fgetc(fptr)) != '\n')
      if(line[len++] == EOF)
	break;
    line[len] = '\0';
  /* push line onto instack */
    for (i=0; i<len; i++)
      push(&instack, line[i]);
  /* send to output while reversing again on outstack */
    for (i=0; i<len; i++) {
      ch = pop(&instack);
      push(&outstack, ch);
      fprintf(stdout, "%c", (char)ch);
    }
    fprintf(stdout, "\n");
  /* pop from outstack and send to output in original order */
    for (i=0; i<len; i++) {
      ch = pop(&outstack);
      fprintf(stdout, "%c", (char)ch);
    }
    fprintf(stdout, "\n");
  }
  return 0;
}
#endif
